#ifndef __TESTPLUGIN_H_INCLUDED
#define __TESTPLUGIN_H_INCLUDED

// You need to define this before the includes, or use /D __SUPPORT_PLUGIN__=1 
// in the C/C++ / Commandline project configuration
#define __SUPPORT_PLUGIN__ 1

#include "IPlugin.h"
#include "IRBRGame.h"
#include "PluginHelpers.h"

#include "Gendef.h" // 2D Gfx header
#include <stdio.h>

const unsigned int NUM_SELECTIONS = 2;

//------------------------------------------------------------------------------------------------//
/**
 * This is the class we will be returning in RBR_CreatePlugin.
 * The class you return needs to inherit the IPlugin interface
 * and fully implement all its functions.
 *
 * This example allows the player to start any stage using the Subaru 03 car
 * and logs the checkpoints and finish times to a text file in the games root directory.
 */
class CTestPlugin : public IPlugin
{
public:
	//------------------------------------------------------------------------------------------------//
	/**
	 * Construct the plugin.
	 * Our example keeps the game interface pointer and creates a fresh log file.
	 */
	CTestPlugin	( IRBRGame* pGame )	
		:	m_pGame	( pGame )
		,	m_iSelection( 0 )	// Reset our selection index
		,	m_iMap	( 10 )		// Set a starting map default selection
	{

		// Create the log file using standard C io.
		FILE *fp = fopen( "rbrplugin.log", "wt" );
		
		// If the file was opened succesfully
		if( fp )
		{
			// Clear it and close it, creating an empty file for appending
			// later
			fflush( fp );
			fclose( fp );
		}
	}

	//------------------------------------------------------------------------------------------------//
	virtual ~CTestPlugin( void )	
	{
		// We have nothing dynamic to delete
	}

	//------------------------------------------------------------------------------------------------//
	virtual const char* GetName( void )	
	{
		// Return the name of the plugin as a standard null terminated string.
		// We call ourself RBRTestPlugin...
		return "RBRTestPlugin";
	}

	//------------------------------------------------------------------------------------------------//
	virtual void DrawResultsUI( void )
	{
		// Race is finished, so we display a nice-looking page

		// Set the drawing colour to the predefined heading colour.
		m_pGame->SetMenuColor( IRBRGame::MENU_HEADING );	
		// Use the large font
		m_pGame->SetFont  ( IRBRGame::FONT_BIG );
		// Write the heading to the screen
		m_pGame->WriteText( 130.0f, 49.0f, "Results" );

		// Set the font to the small font, and use the predefined standard
		// text colour.
		m_pGame->SetFont  ( IRBRGame::FONT_SMALL );
		m_pGame->SetMenuColor( IRBRGame::MENU_TEXT );
		
		if( m_fResults[ 2 ] <= 0.0f )
		{
			// If we didn't finish properly, draw "DNF to the screen.
			m_pGame->WriteText( 200.0f, 100.0f, "DNF" );
		}
		else
		{
			// Format a few strings and display the players result
			char txtCP1[ 32 ];			
			char txtTimeString[ 32 ];
			char txtBuffer[ 128 ];

			sprintf( txtBuffer, "Stage Result for \"%s\" ", m_szPlayerName );
			m_pGame->WriteText( 130.0f, 100.0f, txtBuffer );
			sprintf( txtBuffer, "CheckPoint1 = %s", NPlugin::FormatTimeString( txtCP1, m_fResults[ 0 ] ) );
			m_pGame->WriteText( 130.0f, 125.0f, txtBuffer );
			sprintf( txtBuffer, "CheckPoint2 = %s", NPlugin::FormatTimeString( txtCP1, m_fResults[ 1 ] ) );
			m_pGame->WriteText( 130.0f, 150.0f, txtBuffer );
			sprintf( txtBuffer, "Finish = %s", NPlugin::FormatTimeString( txtTimeString, m_fResults[ 2 ] ) );
			m_pGame->WriteText( 130.0f, 175.0f, txtBuffer );
		}

	}

	//------------------------------------------------------------------------------------------------//
	virtual void DrawFrontEndPage( void )
	{
		// Draw blackout (coordinates specify the 'window' where you don't want black)
		m_pGame->DrawBlackOut( 300.0f, 0.0f, 300.0f, 480.0f );

		// Draw heading
		m_pGame->SetMenuColor( IRBRGame::MENU_HEADING );	
		m_pGame->SetFont( IRBRGame::FONT_BIG );
		m_pGame->WriteText( 73.0f, 49.0f, "Plugin Page" );

		// -2.0f is a good y offset for selection
		m_pGame->DrawSelection( 0.0f, 68.0f + ( static_cast< float >( m_iSelection ) * 21.0f ), 280.0f );

		const char* aszSelections[] =
		{
			"Start Stage",
			"Map",
			// "Exit",
		};

		m_pGame->SetMenuColor( IRBRGame::MENU_TEXT );
		for( unsigned int i = 0; i < NUM_SELECTIONS; ++i )
		{
			if( i == 1 )
			{
				char szStuff[ 128 ];

				if( NPlugin::GetStageName( m_iMap ) )
				{
					sprintf( szStuff, "Map: %s", NPlugin::GetStageName( m_iMap ) );
				}
				else
				{
					sprintf( szStuff, "Map: %d", m_iMap );
				}

				m_pGame->WriteText( 73.0f, 70.0f + ( static_cast< float >( i ) * 21.0f ), szStuff );
			}
			else
			{
				m_pGame->WriteText( 73.0f, 70.0f + ( static_cast< float >( i ) * 21.0f ), aszSelections[ i ] );
			}
		}
		
		// m_pGame->DrawBox( GEN_BOX_LOGOS_CITROEN, 400.0f, 300.0f );
	}

	//------------------------------------------------------------------------------------------------//

	virtual void HandleFrontEndEvents( char txtKeyboard, bool bUp, bool bDown, 
		bool bLeft, bool bRight, bool bSelect )
	{
		if( bSelect )
		{
			if( m_iSelection == 0 && NPlugin::GetStageName( m_iMap ) )
			{
				// Start the game using the map ID in m_iMap. In this example
				// we always use good weather, and the player always get
				// the subaru 03 with dry gravel tires. ( fun on snow stages :-) )
				m_pGame->StartGame( m_iMap, 5, IRBRGame::GOOD_WEATHER, 
					IRBRGame::TYRE_GRAVEL_DRY, NULL );
			}
		}

		if( bUp )
		{
			--m_iSelection;
			if( m_iSelection < 0 )
				m_iSelection = NUM_SELECTIONS - 1;
		}
		
		if( bDown )
		{
			++m_iSelection;
			if( m_iSelection >= NUM_SELECTIONS )
				m_iSelection  = 0;
		}

		if( bLeft && m_iSelection == 1 )
		{
			while( !NPlugin::GetStageName( --m_iMap ) )
			{
				if( m_iMap < 0 )
					m_iMap = 99;
			}
		}

		if( bRight && m_iSelection == 1 )
		{
			while( !NPlugin::GetStageName( ++m_iMap ) )
			{
				if( m_iMap > 99 )
					m_iMap = 0;
			}
		}
	}

	//------------------------------------------------------------------------------------------------//
	virtual void TickFrontEndPage( float fTimeDelta )
	{
		// We have no animations that needs ticking, so we do nothing.
	}

	//------------------------------------------------------------------------------------------------//
	/// Is called when the player timer starts (after GO! or in case of a false start)
	virtual void			StageStarted		( int iMap, 
												  const char* ptxtPlayerName,
												  bool bWasFalseStart )
	{
		// Open the log file in append text mode
		FILE *fp = fopen( "rbrplugin.log", "at" );

		if( fp )
		{
			// Write a start notice
			fprintf( fp, "Stage [%d] started. [%s] %s\n", iMap, ptxtPlayerName, bWasFalseStart ? "False Start" : "" );
			fclose( fp );
		}
        
		// For demonstration purposes, write a message on the screen as well.
		m_pGame->SetColor( 1.0f, 1.0f, 1.0f, 1.0f );
		m_pGame->WriteGameMessage( "Stage Started", 10.0f, 50.0f, 150.0f );
	}

	//------------------------------------------------------------------------------------------------//
	/// Is called when player finishes stage ( fFinishTime is 0.0f if player failed the stage )
	virtual void			HandleResults		( float fCheckPoint1, 
												  float fCheckPoint2,
												  float fFinishTime,
												  const char* ptxtPlayerName )
	{
		// Append the information to the log file.
		FILE *fp = fopen( "rbrplugin.log", "at" );

		if( fp )
		{
			m_fResults[ 0 ] = fCheckPoint1;
			m_fResults[ 1 ] = fCheckPoint2;
			m_fResults[ 2 ] = fFinishTime;
			strncpy( m_szPlayerName, ptxtPlayerName, 32 );

			if( fFinishTime > 0.0f )
			{
				char txtCP1[ 32 ];
				char txtCP2[ 32 ];
				char txtTimeString[ 32 ];
				fprintf( fp, "Stage Result for \"%s\": \n[CP1] = %s\n[CP2] = %s\n[Finish] = %s\n\n", ptxtPlayerName, 
					NPlugin::FormatTimeString( txtCP1, fCheckPoint1 ),
					NPlugin::FormatTimeString( txtCP2, fCheckPoint2 ),
					NPlugin::FormatTimeString( txtTimeString, fFinishTime ) );
			}
			else
			{
				fprintf( fp, "Stage Result for \"%s\": DNF.\n", ptxtPlayerName );
			}

			fclose( fp );
		}
	}

	//------------------------------------------------------------------------------------------------//
	/// Is called when a player passed a checkpoint 
	virtual void			CheckPoint			( float fCheckPointTime,
												  int   iCheckPointID,
												  const char* ptxtPlayerName )
	{
		// Write the checkpoint time to a log file
		FILE *fp = fopen( "rbrplugin.log", "at" );

		if( fp )
		{
			char txtTimeString[ 32 ];
			fprintf( fp, "[CheckPoint %d] : %s : %s\n", iCheckPointID, NPlugin::FormatTimeString( txtTimeString, fCheckPointTime ), ptxtPlayerName );
			fclose( fp );
		}

		// Display an information box on the screen
		m_pGame->SetColor( 1.0f, 1.0f, 1.0f, 1.0f );
		m_pGame->WriteGameMessage( "CHECKPOINT!!!!!!!!!!", 5.0f, 50.0f, 250.0f );
	}

private:
	IRBRGame*		m_pGame;		// Stores the pointer to the game interface.
	int				m_iSelection;
	int				m_iMap;

	float			m_fResults[ 3 ];
	char			m_szPlayerName[ 32 ];
};

#endif